@@ -20,6 +20,14 @@ monitor_host_opts = [
     cfg.IntOpt('monitoring_interval',
                default=60,
                help='Monitoring interval(in seconds) of node status.'),
+    cfg.IntOpt('monitoring_samples',
+               default=1,
+               help='''
+Monitoring probes to collect before making the decision to send Masakari
+notification about the node status. If and only if ``monitoring_samples``
+consecutive reports have the same status, will the Masakari notification
+be sent.
+'''),
     cfg.IntOpt('api_retry_max',
                default=12,
                help='Number of retries for send a notification in'
@@ -79,6 +87,17 @@ like '5405,5406'.
 The number of port numbers you specify must be equal to the number of
 corosync_multicast_interfaces values and must be in correct order with
 relevant interfaces in corosync_multicast_interfaces.
+'''),
+    cfg.StrOpt('pacemaker_node_type',
+               default='autodetect',
+               choices=('autodetect', 'cluster', 'remote'),
+               help='''
+Using this option, one can avoid systemd checks that would establish whether
+this hostmonitor is running alongside Corosync and Pacemaker (the ``cluster``
+stack) or Pacemaker Remote (the ``remote`` stack).
+
+The default (``autodetect``) ensures backward compatibility and means systemd
+is used to check the stack.
 '''),
 ]
 
