@@ -14,6 +14,7 @@
 
 import socket
 
+from collections import deque
 import eventlet
 from oslo_log import log as oslo_logging
 from oslo_utils import timeutils
@@ -56,6 +57,27 @@ class HandleHost(driver.DriverBase):
         self.crmmon_xml_parser = parse_crmmon_xml.ParseCrmMonXml()
         self.status_holder = hold_host_status.HostHoldStatus()
         self.notifier = masakari.SendNotification()
+        self.monitoring_data = {}
+
+    def _update_monitoring_data(self, hostname, status):
+        health_history = self.monitoring_data.setdefault(
+            hostname, deque([], maxlen=CONF.host.monitoring_samples))
+        health_history.append(status)
+
+    def get_stabilised_host_status(self, hostname):
+        health_history = self.monitoring_data.get(hostname)
+        if len(health_history) < CONF.host.monitoring_samples:
+            LOG.debug("Not enough monitoring data for host %s.", hostname)
+            return '_being_collected'
+
+        stabilised_status = health_history[0]
+
+        # If and only if the sequence of host status is consistently the same,
+        # will it return that status.
+        if len(health_history) == health_history.count(stabilised_status):
+            return stabilised_status
+        else:
+            return '_uncertain'
 
     def _check_pacemaker_services(self, target_service):
         try:
@@ -79,11 +101,19 @@ class HandleHost(driver.DriverBase):
         :returns: 0 if normal, 1 if abnormal, 2 if configuration file is
             wrong or neither pacemaker nor pacemaker-remote is running.
         """
-        # Check whether the pacemaker services is normal.
-        corosync_status = self._check_pacemaker_services('corosync')
-        pacemaker_status = self._check_pacemaker_services('pacemaker')
-        pacemaker_remote_status = self._check_pacemaker_services(
-            'pacemaker_remote')
+        if CONF.host.pacemaker_node_type == 'autodetect':
+            # Check whether the Corosync/Pacemaker services are running.
+            corosync_status = self._check_pacemaker_services('corosync')
+            pacemaker_status = self._check_pacemaker_services('pacemaker')
+            pacemaker_remote_status = self._check_pacemaker_services(
+                'pacemaker_remote')
+        else:
+            corosync_status = (
+                CONF.host.pacemaker_node_type == 'cluster')
+            pacemaker_status = (
+                CONF.host.pacemaker_node_type == 'cluster')
+            pacemaker_remote_status = (
+                CONF.host.pacemaker_node_type == 'remote')
 
         if corosync_status is False or pacemaker_status is False:
             if pacemaker_remote_status is False:
@@ -287,8 +317,8 @@ class HandleHost(driver.DriverBase):
             if hostname == self.my_hostname:
                 continue
 
-            # Get current status and old status.
             current_status = node_state_tag.get('crmd')
+            self._update_monitoring_data(hostname, current_status)
             old_status = self.status_holder.get_host_status(hostname)
 
             # If old_status is None, This is first get of host status.
@@ -300,21 +330,28 @@ class HandleHost(driver.DriverBase):
                 self.status_holder.set_host_status(node_state_tag)
                 continue
 
+            stabilised_status = self.get_stabilised_host_status(hostname)
+
             # Output host status.
-            msg = ("'%s' is '%s'.") % (hostname, current_status)
+            msg = ("'%s' is '%s' (current: '%s').") % (hostname,
+                                                       stabilised_status,
+                                                       current_status)
             LOG.info("%s", msg)
 
-            # If host status changed, send a notification.
-            if current_status != old_status:
-                if current_status != 'online' and current_status != 'offline':
-                    # If current_status is not 'online' or 'offline',
+            if stabilised_status == '_being_collected':
+                continue
+
+            # If host stabilised status changed, send a notification.
+            if stabilised_status != old_status:
+                if stabilised_status not in ['online', 'offline']:
+                    # If stabilised_status is not 'online' or 'offline',
                     # hostmonitor doesn't send a notification.
                     msg = ("Since host status is '%s',"
                            " hostmonitor doesn't send a notification.") \
-                        % current_status
+                        % stabilised_status
                     LOG.info("%s", msg)
                 else:
-                    event = self._make_event(hostname, current_status)
+                    event = self._make_event(hostname, stabilised_status)
 
                     # Send a notification.
                     self.notifier.send_notification(
@@ -322,8 +359,9 @@ class HandleHost(driver.DriverBase):
                         CONF.host.api_retry_interval,
                         event)
 
-            # Update host status.
-            self.status_holder.set_host_status(node_state_tag)
+                if stabilised_status != '_uncertain':
+                    # Update host status.
+                    self.status_holder.set_host_status(node_state_tag)
 
     def _check_host_status_by_crm_mon(self):
         crmmon_xml = self._get_crmmon_xml()
@@ -386,9 +424,10 @@ class HandleHost(driver.DriverBase):
 
         This method monitors hosts.
         """
-        self.running = True
-        while self.running:
-            try:
+        try:
+            self.running = True
+            while self.running:
+
                 # Check whether corosync communication between hosts
                 # is normal.
                 ret = self._check_hb_line()
@@ -401,11 +440,16 @@ class HandleHost(driver.DriverBase):
                     eventlet.greenthread.sleep(CONF.host.monitoring_interval)
                     continue
 
+                if CONF.host.pacemaker_node_type == 'autodetect':
+                    pacemaker_remote_status = self._check_pacemaker_services(
+                        'pacemaker_remote')
+                else:
+                    pacemaker_remote_status = (
+                        CONF.host.pacemaker_node_type == 'remote')
+
                 # Check the host status is stable or unstable by crmadmin.
                 # It only checks when this process runs on the full cluster
                 # stack of corosync.
-                pacemaker_remote_status = self._check_pacemaker_services(
-                    'pacemaker_remote')
                 if pacemaker_remote_status is False:
                     if self._check_host_status_by_crmadmin() != 0:
                         LOG.warning("hostmonitor skips monitoring hosts.")
@@ -422,7 +466,9 @@ class HandleHost(driver.DriverBase):
                 if status_func() != 0:
                     LOG.warning("hostmonitor skips monitoring hosts.")
 
-            except Exception as e:
-                LOG.exception("Exception caught: %s", e)
+                eventlet.greenthread.sleep(CONF.host.monitoring_interval)
 
-            eventlet.greenthread.sleep(CONF.host.monitoring_interval)
+        except Exception as e:
+            LOG.exception("Exception caught: %s", e)
+
+        return
